from pairs.ir.types import Types
from pairs.code_gen.cgen import CGen
from pairs.code_gen.target import Target
from pairs.sim.domain_partitioners import DomainPartitioners
from pairs.sim.shapes import Shapes
from pairs.sim.simulation import Simulation


def simulation(
    ref,
    shapes,
    dims=3,
    double_prec=False,
    use_contact_history=False,
    particle_capacity=800000,
    neighbor_capacity=20,
    debug=False):

    return Simulation(
        CGen(ref, debug), shapes, dims, double_prec, use_contact_history, particle_capacity, neighbor_capacity)

def target_cpu(parallel=False):
    if parallel:
        return Target(Target.Backend_CPP, [Target.Feature_CPU, Target.Feature_OpenMP])

    return Target(Target.Backend_CPP, Target.Feature_CPU)

def target_gpu():
    return Target(Target.Backend_CUDA, Target.Feature_GPU)

def int32():
    return Types.Int32

def float():
    return Types.Float

def double():
    return Types.Double

def real():
    return Types.Real

def vector():
    return Types.Vector

def matrix():
    return Types.Matrix

def quaternion():
    return Types.Quaternion

def point_mass():
    return Shapes.PointMass

def sphere():
    return Shapes.Sphere

def box():
    return Shapes.Box

def halfspace():
    return Shapes.Halfspace

def regular_domain_partitioner():
    return DomainPartitioners.Regular

def regular_domain_partitioner_xy():
    return DomainPartitioners.RegularXY

def block_forest():
    return DomainPartitioners.BlockForest
